import numpy as np
import cv2
import sys

#selected font for text
font = cv2.FONT_HERSHEY_SIMPLEX

#selected img num for test
test_number = 1
path = "test_color/"


#define colors BGR 
black = (43,42,44)

gray = (110,110,110)

yellow = (58,130,156)

orange = (61,85,170)

red = (44,39,100)

blue = (134,96,80)

green = (49,80,42)

#make array of colors and names
color = [black,gray,yellow,orange,red,blue,green]
color_name = ["black","gray","yellow","orange","red","blue","green"]


e1 = cv2.getTickCount()

#open selected image in BGR
img = cv2.imread(str(path)+"samples/test_"+str(test_number)+".jpg",1)

#copy img into grayscale
img_gray=cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)

#use median blur on grayscale image
img_gray=cv2.medianBlur(img_gray,21)


#SELECT THRESHOLD

#adaptive for uneven light condition
thres = cv2.adaptiveThreshold(img_gray,255,cv2.ADAPTIVE_THRESH_MEAN_C, cv2.THRESH_BINARY_INV,121,10)

#show only for testing
cv2.imshow('treshold_before_morph',thres)

#rectangle kernel
kernel_size =  30

kernel = cv2.getStructuringElement(cv2.MORPH_RECT,(kernel_size, kernel_size))

#applicate morphological closing, border replicate keep border non touching objects non touching
thres = cv2.morphologyEx(thres, cv2.MORPH_CLOSE, kernel, borderType = cv2.BORDER_REPLICATE)

# Find Contours, only external and all pixels no approx.
contours, _ = cv2.findContours(thres, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE )


#draw and store results outside time measument

for cnt in range (0,len(contours)):

    area = cv2.contourArea(contours[cnt])
    #if object is big enough cal position
    if(area > 4000):

        #cal pos. using moments
        M = cv2.moments(contours[cnt])
        cX = int(M["m10"] / M["m00"])
        cY = int(M["m01"] / M["m00"])

        #mask for object
        mask = np.zeros(img.shape[:2], np.uint8)
        cv2.drawContours(mask,[contours[cnt]],-1,255,-1)

        #mean under mask in bgr image
        mean = cv2.mean(img,mask)[:-1]
        mean = np.around(mean, decimals = 0)

        
        diff=[]
        #cal distances from defined colors
        for i in range(0,len(color)):
            diff.append(abs(color[i]-mean)[0] + abs(color[i]-mean)[1] + abs(color[i]-mean)[2])

        #find minimal distance from color and coresponding name
        color_text = color_name[diff.index(min(diff))]
        
        #Draw text of color
        cv2.drawContours(img, contours, cnt, (0, 255, 0), 1)

        cv2.putText(img, color_text, (cX,cY), font, 0.8, (255, 255,255), 2, cv2.LINE_AA)





#end time measurement
e2 = cv2.getTickCount()
time = (e2 - e1)/ cv2.getTickFrequency()
print("Time: ",time," s")



#save result image
cv2.imwrite(path+"results/result_"+str(test_number)+".jpg",img)

#show images
#cv2.imshow('img_gray',img_gray)
#cv2.imshow('treshold',thres)
cv2.imshow('kontury',img)
        
#wait for key and then exit program
cv2.waitKey(0)
cv2.destroyAllWindows()

sys.exit()
